unit TRakBinaryBlobExampleMain;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  StdCtrls, ComCtrls, RakBinaryStreamData;

type

  TForm1 = class(TForm)
    GroupBox1: TGroupBox;
    Label1: TLabel;
    Label2: TLabel;
    CckBool: TCheckBox;
    GroupBox2: TGroupBox;
    BtnImport: TButton;
    RedFile: TRichEdit;
    Label3: TLabel;
    BtnSaveAs: TButton;
    BtnLoad: TButton;
    EdtText: TEdit;
    CbxZahl: TComboBox;
    OdgFile: TOpenDialog;
    Sdg: TSaveDialog;
    Odg: TOpenDialog;
    procedure FormCreate(Sender: TObject);
    procedure BtnImportClick(Sender: TObject);
    procedure FormDestroy(Sender: TObject);
    procedure BtnSaveAsClick(Sender: TObject);
    procedure BtnLoadClick(Sender: TObject);
  private
    { Private-Deklarationen }
    FFileLoaded: boolean;
    FFileData: TMemoryStream;

    procedure ShowFileData;
  public
    { Public-Deklarationen }
  end;

var
  Form1: TForm1;

implementation

{$R *.DFM}

const
	APP_ID = 'TRakBinaryBlobExample';

type
	TBlobRecord = record
  	Text: string[80];
    Zahl: byte;
    Bool: boolean;
  end;


procedure TForm1.FormCreate(Sender: TObject);
begin
	CbxZahl.ItemIndex := 0;
  FFileLoaded := false;
  FFileData := TMemoryStream.Create;
end;

procedure TForm1.FormDestroy(Sender: TObject);
begin
	FFileData.Free;
end;

procedure TForm1.BtnImportClick(Sender: TObject);
var
	Stream: TFileStream;
  Size: integer;
begin
	if OdgFile.Execute then
  begin
    Screen.Cursor := crHourGlass;
  	try
      Stream := TFileStream.Create(OdgFile.FileName, fmOpenRead);
      try
        Size := Stream.Size;
      finally
        Stream.Free;
      end;

      if Size >= 1024 * 1024 then
      begin
        ShowMessage('Aus Speichergrnden werden nur Dateien mit weniger als 1 MB akzeptiert');
      end
      else
      begin
        FFileData.LoadFromFile(OdgFile.FileName);
        FFileLoaded := true;
        ShowFileData;
      end;
    finally
    	Screen.Cursor := crDefault
    end;
  end;
end;


procedure TForm1.ShowFileData;
var
	EinByte: byte;
  HexString: string;
  HexByte: string;
  i: integer;
begin
	if not FFileLoaded then
		RedFile.Text := ''
  else
  begin
    SetLength(HexString, FFileData.Size * 2);
    FFileData.Position := 0;
    for i := 0 to FFileData.Size - 1 do
    begin
      FFileData.Read(EinByte, 1);
      HexByte := IntToHex(EinByte, 2);
      HexString[i*2] := HexByte[1];
      HexString[i*2+1] := HexByte[2];
    end;
    RedFile.Text := HexString;
  end;
end;

procedure TForm1.BtnSaveAsClick(Sender: TObject);
var
	RecordData: TBlobRecord;
	Data: TRakBinaryStreamData;
  DoIt: boolean;
  FileAttr: integer;
  Blob: TRakBinaryBlob;
begin
	if Sdg.Execute then
  begin
  	DoIt := true;
  	if FileExists(Sdg.FileName) then
    begin // Wenn es bereits eine Datei mit dem gleichen Namen gibt
    	FileAttr := FileGetAttr(Sdg.FileName);
      if (FileAttr and SysUtils.faReadOnly) > 0 then
      begin // Wenn diese Datei schreibgeschtzt ist
        DoIt := MessageDlg('Es gibt bereits eine Datei mit diesem Namen.'
            + #13'Diese Datei ist schreibgeschtzt?'
            + #13'Soll diese berschrieben werden?', mtConfirmation,
            [mbYes, mbNo], 0) = mrYes;

        if DoIt then
        begin	// Wenn diese Datei berschrieben werden soll
        	// Dann versuchen wir den Schreibschutz aufzuheben
	      	if FileSetAttr(Sdg.FileName, FileAttr and not SysUtils.faReadOnly) <> 0 then
          begin // Wenn das nicht geklappt hat
            DoIt := false;
            Application.MessageBox('Der Schreibschutz konnte nicht aufgehoben werden.',
            	'Datei kann nicht geschrieben werden.', MB_OK);
          end;
        end;
      end
      else
      begin // Wenn diese Datei nicht schreibgeschtzt ist
        DoIt := MessageDlg('Es gibt bereits eine Datei mit diesem Namen.'
            + #13'Soll diese berschrieben werden?', mtConfirmation,
            [mbYes, mbNo], 0) = mrYes;
      end;
    end;

    if DoIt then
    begin // Wenn die Datei wirklich gespeichert werden soll
      // Zuerst werden die Inhalte der Eingabefelder in den Record umgeschaufelt
      RecordData.Text := EdtText.Text;
      RecordData.Zahl := CbxZahl.ItemIndex;
      RecordData.Bool := CckBool.Checked;

      // Den Zwischenpuffer erzeugen
      Data := TRakBinaryStreamData.Create;
      try
      	// Den Namen festlegen, an dem wir beim Laden erkennen, dass die Daten wirklich zu uns gehren
        Data.ApplicationName := APP_ID;

        // Das TRakBinaryBlob fr den Record erzeugen
        Blob := TRakBinaryBlob(Data.AddAttribute(TRakBinaryBlob, 'RecordData'));
  			Blob.SetAsBinary(@RecordData, sizeof(RecordData));
        Blob.IsNull := false;

        // Das TRakBinaryBlob fr die Daten aus der Datei erzeugen
        Blob := TRakBinaryBlob(Data.AddAttribute(TRakBinaryBlob, 'FileData'));
        if FFileLoaded then
        begin // Wenn wirklich eine Datei geladen wurde
        	// Nur wenn IsNull = false, werden die Daten wirklich gespeichert
	        Blob.IsNull := false;
          // Die eigentlichen Daten bergeben
	  			Blob.SetAsBinary(FFileData.Memory, FFileData.Size);
        end;

        try
        	// Die Daten in einer Datei speichern
	        Data.SaveToFile(Sdg.FileName);
        except
        	on Err: Exception do
          	begin
            	Application.MessageBox(PChar(Err.Message), 'Datei konnte nicht gespeichert werden.', MB_OK);
            end;
        end;
      finally
        Data.Free;
      end;
    end;
  end;
end;

procedure TForm1.BtnLoadClick(Sender: TObject);
var
	Data: TRakBinaryStreamData;
  Attr: TRakBinaryAttribute;
  Blob: TRakBinaryBlob;
	RecordData: TBlobRecord;
begin
	if Odg.Execute then
  begin
    Data := TRakBinaryStreamData.Create;
    Screen.Cursor := crHourGlass;
    try
      case Data.LoadFromFile(Odg.FileName) of
        blrOK:
        	begin
          	if Data.ApplicationName = APP_ID then
            begin // Wenn die Daten wirklich fr uns bestimmt sind
            	// Alles erst einmal leeren
            	EdtText.Text := '';
              CbxZahl.ItemIndex := 0;
              CckBool.Checked := false;
              FFileLoaded := false;
              FFileData.Size := 0;

              // Hier lesen wir die Daten, die aus den Eingabefeldern stammten
            	Attr := Data.AttributeByName('RecordData');
              if Attr is TRakBinaryBlob then
              begin // Wenn es die Daten gibt und vom erwarteten Typ sind
              	Blob := TRakBinaryBlob(Attr);
                if not Blob.IsNull and (Blob.GetBinarySize = sizeof(RecordData)) then
                begin // Wenn die erwartete Anzahl von Bytes in dem Blob enthalten sind
                	// Die Daten in die Record-Variable einlesen
                	Blob.GetAsBinary(@RecordData, sizeof(RecordData));

                  // und an die Eingabefelder bergeben
                  EdtText.Text := RecordData.Text;
                  CbxZahl.ItemIndex := RecordData.Zahl;
                  CckBool.Checked := RecordData.Bool;
                end;
              end;

              // Hier werden die Daten aus der Datei gelesen
            	Attr := Data.AttributeByName('FileData');
              if Attr is TRakBinaryBlob then
              begin // Wenn es die Daten gibt und vom erwarteten Typ sind
              	Blob := TRakBinaryBlob(Attr);
                FFileLoaded := not Blob.IsNull;
                if FFileLoaded then
                begin // Wenn wirklich Daten gespeichert sind
                	// Den Puffer auf die richtige Gre bringen
                	FFileData.Size := Blob.GetBinarySize;
                	// Die Daten aus dem Blob in den Puffer kopieren
                	Blob.GetAsBinary(FFileData.Memory, FFileData.Size);
                end;
              end;

              // und anzeigen
              ShowFileData;
            end
            else
            begin
		        	Application.MessageBox('Die Datei wurde nicht von dieser Anwendung erzeugt.', 'Datei konnte nicht geffnet werden', MB_OK);
            end;
          end;
        blrErrVersion:
        	Application.MessageBox('Es ist zwar das richtige Datenformat, aber in einer Version, die nicht untersttzt wird.', 'Datei konnte nicht geffnet werden', MB_OK);
        blrErrFormat:
        	Application.MessageBox('Die Datei enthlt kein bekanntes Datenformat.', 'Datei konnte nicht geffnet werden', MB_OK);
        blrErrCorrupt:
        	Application.MessageBox('Die Daten im Stream sind defekt.', 'Datei konnte nicht geffnet werden', MB_OK);
        else
        	Application.MessageBox('Unbekannter Rckgabeparameter.', 'Datei konnte nicht geffnet werden', MB_OK);
      end; // case
    finally
    	Screen.Cursor := crDefault;
    	Data.Free;
    end;
  end;
end;

end.
